//######################################################//
//## Timothy Lottes CRT shader.(Ported by Asmodean)####//

/*
[configuration]

[OptionBool]
GUIName = Lottes CRT
OptionName = A_LOTTES_CRT
DefaultValue = true

[OptionRangeInteger]
GUIName = MaskingType
OptionName = A_MASK_TYPE
MinValue = 0
MaxValue = 3
StepAmount = 1
DefaultValue = 1
DependentOption = A_LOTTES_CRT

[OptionRangeFloat]
GUIName = ScanBrightness
OptionName = B_CRT_BRIGHTNESS
MinValue = -16.00
MaxValue = 0.00
StepAmount = 0.01
DefaultValue = -8.00
DependentOption = A_LOTTES_CRT

[OptionRangeFloat]
GUIName = FilterAmount
OptionName = B_CRT_FILTER
MinValue = -4.00
MaxValue = 0.00
StepAmount = 0.01
DefaultValue = -1.00
DependentOption = A_LOTTES_CRT

[OptionRangeFloat]
GUIName = HorizontalWarp
OptionName = F_H_WARP
MinValue = 0.000
MaxValue = 0.100
StepAmount = 0.001
DefaultValue = 0.000
DependentOption = A_LOTTES_CRT

[OptionRangeFloat]
GUIName = VerticalWarp
OptionName = G_V_WARP
MinValue = 0.000
MaxValue = 0.100
StepAmount = 0.001
DefaultValue = 0.000
DependentOption = A_LOTTES_CRT

[OptionRangeFloat]
GUIName = MaskAmountDark
OptionName = H_MASK_DARK
MinValue = 0.00
MaxValue = 2.00
StepAmount = 0.01
DefaultValue = 0.50
DependentOption = A_LOTTES_CRT

[OptionRangeFloat]
GUIName = MaskAmountLight
OptionName = I_MASK_LIGHT
MinValue = 0.00
MaxValue = 4.00
StepAmount = 0.01
DefaultValue = 1.50
DependentOption = A_LOTTES_CRT

[OptionRangeFloat]
GUIName = ResolutionReduction
OptionName = Z_RES_DIVISOR
MinValue = 1.00
MaxValue = 6.00
StepAmount = 0.01
DefaultValue = 1.10
DependentOption = A_LOTTES_CRT

[/configuration]
*/

/*------------------------------------------------------------------------------
                            [GLOBALS|FUNCTIONS]
------------------------------------------------------------------------------*/

float2 texcoord = GetCoordinates();
float2 screenSize = GetResolution();
float2 pixelSize = GetInvResolution();
float2 texSize = textureSize(samp9, 0);

/*------------------------------------------------------------------------------
                           [LOTTES CRT CODE SECTION]
------------------------------------------------------------------------------*/

float2 res = screenSize / GetOption(Z_RES_DIVISOR);

float ToLinear1(float c)
{
    return(c <= 0.04045) ? c / 12.92 : pow((c + 0.055) / 1.055, 2.4);
}

float3 ToLinear(float3 c)
{
    return float3(ToLinear1(c.r), ToLinear1(c.g), ToLinear1(c.b));
}

float ToSrgb1(float c)
{
    return(c < 0.0031308 ? c * 12.92 : 1.055 * pow(c, 0.41666) -0.055);
}

float3 ToSrgb(float3 c)
{
    return float3(ToSrgb1(c.r), ToSrgb1(c.g), ToSrgb1(c.b));
}

float3 Fetch(float2 pos, float2 off)
{
    pos = floor(pos * res + off) / res;
    if(max(abs(pos.x - 0.5), abs(pos.y - 0.5)) > 0.5) { return float3(0.0, 0.0, 0.0); }

    return ToLinear(SampleLocation(pos.xy).rgb);
}

float2 Dist(float2 pos)
{
    pos = (pos * res);
    return -((pos - floor(pos)) - float2(0.5, 0.5));
}

float Gaus(float pos, float scale)
{
    return exp2(scale * pos * pos);
}

float3 Horz3(float2 pos,float off)
{
    float3 b = Fetch(pos,float2(-1.0, off));
    float3 c = Fetch(pos,float2( 0.0, off));
    float3 d = Fetch(pos,float2( 1.0, off));
    float dst = Dist(pos).x;

    // Convert distance to weight.
    float scale = GetOption(B_CRT_FILTER);
    float wb=Gaus(dst-1.0, scale);
    float wc=Gaus(dst+0.0, scale);
    float wd=Gaus(dst+1.0, scale);

    return (b*wb+c*wc+d*wd)/(wb+wc+wd);
}

float3 Horz5(float2 pos,float off)
{
    float3 a = Fetch(pos, float2(-2.0, off));
    float3 b = Fetch(pos, float2(-1.0, off));
    float3 c = Fetch(pos, float2( 0.0, off));
    float3 d = Fetch(pos, float2( 1.0, off));
    float3 e = Fetch(pos, float2( 2.0, off));
    float dst=Dist(pos).x;

    // Convert distance to weight.
    float scale = GetOption(B_CRT_FILTER);

    float wa=Gaus(dst-2.0, scale);
    float wb=Gaus(dst-1.0, scale);
    float wc=Gaus(dst+0.0, scale);
    float wd=Gaus(dst+1.0, scale);
    float we=Gaus(dst+2.0, scale);

    return (a*wa+b*wb+c*wc+d*wd+e*we)/(wa+wb+wc+wd+we);
}

// Return scanline weight.
float Scan(float2 pos,float off)
{
    float dst = Dist(pos).y;
    return Gaus(dst+off, GetOption(B_CRT_BRIGHTNESS));
}

float3 Tri(float2 pos)
{
    float3 a = Horz3(pos,-1.0);
    float3 b = Horz5(pos, 0.0);
    float3 c = Horz3(pos, 1.0);

    float wa = Scan(pos,-1.0);
    float wb = Scan(pos, 0.0);
    float wc = Scan(pos, 1.0);

    return a*wa+b*wb+c*wc;
}

float2 Warp(float2 pos)
{
    pos = pos * 2.0-1.0;    
    pos *= float2(1.0 + (pos.y*pos.y) * (GetOption(F_H_WARP)), 1.0 + (pos.x*pos.x) * (GetOption(G_V_WARP)));
    return pos * 0.5+0.5;
}

float3 Mask(float2 pos)
{
    if (GetOption(A_MASK_TYPE) == 0) {
    // Very compressed TV style shadow mask.
    float line = GetOption(I_MASK_LIGHT);
    float odd = 0.0;

    if(frac(pos.x/6.0) < 0.5)odd=1.0;
    if(frac((pos.y+odd)/2.0) < 0.5)line=GetOption(H_MASK_DARK);  
    pos.x = frac(pos.x/3.0);
    float3 mask = float3(GetOption(H_MASK_DARK), GetOption(H_MASK_DARK), GetOption(H_MASK_DARK));

    if(pos.x < 0.333) mask.r = GetOption(I_MASK_LIGHT);
    else if(pos.x < 0.666)mask.g = GetOption(I_MASK_LIGHT);
    else mask.b = GetOption(I_MASK_LIGHT);
    mask *= line;

    return mask; }
    
    if (GetOption(A_MASK_TYPE) == 1) {
    // Aperture-grille.
    pos.x = frac(pos.x/3.0);
    float3 mask = float3(GetOption(H_MASK_DARK), GetOption(H_MASK_DARK), GetOption(H_MASK_DARK));

    if(pos.x < 0.333)mask.r = GetOption(I_MASK_LIGHT);
    else if(pos.x < 0.666)mask.g = GetOption(I_MASK_LIGHT);
    else mask.b = GetOption(I_MASK_LIGHT);

    return mask; }
    
    if (GetOption(A_MASK_TYPE) == 2) {
    // Stretched VGA style shadow mask (same as prior shaders).
    pos.x += pos.y*3.0;
    float3 mask = float3(GetOption(H_MASK_DARK), GetOption(H_MASK_DARK), GetOption(H_MASK_DARK));
    pos.x = frac(pos.x/6.0);

    if(pos.x < 0.333)mask.r = GetOption(I_MASK_LIGHT);
    else if(pos.x < 0.666)mask.g = GetOption(I_MASK_LIGHT);
    else mask.b = GetOption(I_MASK_LIGHT);

    return mask; }
    
    if (GetOption(A_MASK_TYPE) == 3) {
    // VGA style shadow mask.
    pos.xy = floor(pos.xy*float2(1.0, 0.5));
    pos.x += pos.y*3.0;

    float3 mask = float3(GetOption(H_MASK_DARK), GetOption(H_MASK_DARK), GetOption(H_MASK_DARK));
    pos.x = frac(pos.x/6.0);

    if(pos.x < 0.333)mask.r = GetOption(I_MASK_LIGHT);
    else if(pos.x < 0.666)mask.g = GetOption(I_MASK_LIGHT);
    else mask.b=GetOption(I_MASK_LIGHT);
    return mask; }
}

float4 LottesCRT(float4 color)
{
    float2 pos = Warp(texcoord);
    color.rgb = Tri(pos) * Mask(gl_FragCoord.xy);
    color.rgb = ToSrgb(color.rgb);
    color.a = 1.0;

    return color;
}

/*------------------------------------------------------------------------------
                     [MAIN() & COMBINE PASS CODE SECTION]
------------------------------------------------------------------------------*/

void main()
{
    float4 color = Sample();

    if (OptionEnabled(A_LOTTES_CRT)) { color = LottesCRT(color); }

    SetOutput(color);
}